library(ggplot2) #A diagram kirajzolásához
library(ggpubr) #Csak a sokaság hisztogramjának kirajzolásához használtam
library(gridExtra) #A két diagram egymás mellé rajzolásához szükséges

#Sokaság létrehozása
sok1 <- rnorm(200, mean = 10, sd = 2)
sok2 <- rnorm(200, mean = 25, sd = 5)
sok3 <- rnorm(200, mean = 30, sd = 1.5)
sok4 <- rnorm(200, mean = 40, sd = 3)
sok5 <- rnorm(200, mean = 50, sd = 2)

sokasag <- c(sok1, sok2, sok3, sok4, sok5)

#Sokaság hisztogramja
gghistogram(sokasag, bins = 32)

#Sokaság átlagának és szórásának kiszámítása
sokasagAtlag <- mean(sokasag)
sokasagSzoras <- sd(sokasag)

#Kétféle minta létrehozása
minta1 <- rnorm(100, mean = 29, sd = 3)  #mintaátlag a megbízhatósági tartományban 
minta2 <- rnorm(100, mean = 35, sd = 3)  #mintaátlag a megbízhatósági tartományon kívül

#mintaátlagok kiszámítása
minta1Atlag <- mean(minta1)
minta2Atlag <- mean(minta2)

#A kétféle grafikon elkészítése és ábrázolása egymás mellett
plot1 <- z.testgraph(mintaAtlag = minta1Atlag, sokasagAtlag = sokasagAtlag, sokasagSzoras = sokasagSzoras, n = 100)
plot2 <- z.testgraph(mintaAtlag = minta2Atlag, sokasagAtlag = sokasagAtlag, sokasagSzoras = sokasagSzoras, n = 100)
grid.arrange(plot1, plot2, ncol = 2)

#grafikon rajzoló függvény
z.testgraph <- function(mintaAtlag, sokasagAtlag, sokasagSzoras, n)
{
  #A mintaátlagok eloszlás paramétereinek kiszámítása
  mintaEloszlasAtlag <- sokasagAtlag
  mintaEloszlasSzoras <- sokasagSzoras/sqrt(n)
  
  #A mintaátlagok 95%-os megbízhatósági határainak kiszámítása
  mintaEloszlas95Lower <- mintaEloszlasAtlag - 2 * mintaEloszlasSzoras
  mintaEloszlas95Upper <- mintaEloszlasAtlag + 2 * mintaEloszlasSzoras
  
  #A diagram kirajzolásához szükséges legkisebb és legnagyobb x-értékek kiszámítása
  diagramLowerLimit <- mintaEloszlasAtlag - 4 * mintaEloszlasSzoras
  diagramUpperLimit <- mintaEloszlasAtlag + 4 * mintaEloszlasSzoras
  
  #A mintaátlag és a megbízhatósági határértékek értékének szöveggé alakítása a tizedesjegyek beállításával
  mintaAtlagSzoveg <- as.character(format(mintaAtlag, digits = 5))
  mintaEloszlas95LowerSzoveg <- as.character(format(mintaEloszlas95Lower, digits = 5))
  mintaEloszlas95UpperSzoveg <- as.character(format(mintaEloszlas95Upper, digits = 5))
  
  #A diagram kirajzolása
  ggplot() + xlim(diagramLowerLimit, diagramUpperLimit) +                                               #Diagramterület
    geom_function(fun = dnorm, args = list(mean = mintaEloszlasAtlag, sd = mintaEloszlasSzoras)) +      #mintaeloszlás függvény
    geom_vline(xintercept = mintaEloszlasAtlag, color = "darkgreen", linetype = "dashed") +             #minta nagyátlag   
    geom_vline(xintercept = mintaEloszlas95Lower, color = "darkgreen", linetype = "dashed") +           #95% megb.tartomány alsó határ
    geom_vline(xintercept = mintaEloszlas95Upper, color = "darkgreen", linetype = "dashed") +           #95% megb.tartomány felső határ
    geom_vline(xintercept = mintaAtlag, color = "red", linetype = "solid", size = 2) +                  #minta átlaga
    labs(title = "Egymintás Z-próba - 95% megbízhatósági szint") +                                      #diagram címe
    annotate(geom = "label", x = mintaAtlag, y = 0.25, label = "minta átlag") +                         #mintaátlag felirat
    annotate(geom = "label", x = mintaEloszlas95Lower, y = 0, label = "95% meg.tart. alsó határ") +     #95% alsó határ felirat
    annotate(geom = "label", x = mintaEloszlas95Upper, y = 0, label = "95% meg.tart. felső határ") +    #95% felső határ felirat
    annotate(geom = "label", x = mintaAtlag, y = 0.23, label = mintaAtlagSzoveg) +                      #mintaátlag érték felirat
    annotate(geom = "label", x = mintaEloszlas95Lower, y = 0.02, label = mintaEloszlas95LowerSzoveg) +  #95% alsó határérték felirat
    annotate(geom = "label", x = mintaEloszlas95Upper, y = 0.02, label = mintaEloszlas95UpperSzoveg)    #95% felső határérték felirat
}
